// This is the code for the lexical-distributional model described in:
//    Feldman, N. H., Griffiths, T. L., Goldwater, S., & Morgan, J. L.  "A 
//    role for the developing lexicon in phonetic category acquisition."  
//    Psychological Review.
// Please cite that paper if you use this code.  You can contact the 
// corresponding author by emailing <nhf@umd.edu>.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version. This program is distributed in the 
// hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
// See the GNU General Public License for more details.  You should have 
// received a copy of the GNU General Public License along with this program.  
// If not, see <http://www.gnu.org/licenses/>.
// 
// Copyright (C) 2013 Naomi Feldman

#include <stdio.h>
#include <assert.h>						// for safer programming
#include <math.h>							// needed for log, exp, and gamma functions
#include <gsl/gsl_matrix.h>		// gnu scientific library - matrices
#include <gsl/gsl_linalg.h>		// gnu scientific library - determinant
#include <gsl/gsl_blas.h>
#include <gsl/gsl_cblas.h>		// some vector functions
#include <stdlib.h>

// Some Constants
#define NUM_WORDS 5000        // total number of words in corpus
#define NUM_PHONES 6409       // total number of vowels in corpus
#define NUM_CONSONANTS 24     // number of consonants in dataset
#define LONGEST 5					    // length of the longest word in the dataset

#define myrand() (double) (((unsigned long) randomMT()) / 4294967296.)
//#define myrand() (double) (rand()/(RAND_MAX+1.0))

// More Constants
const int MAX_ITER = 10000; // number of iterations 10000
const int LIKELIHOOD_FREQ = 100; // likelihood is reported every LIKELIHOOD_FREQ iterations
const double ANNEALING_CONSTANT = 9.; // annealing constant
const int DIM = 2; // number of dimensions
const int NUM_LEXICAL_SAMPLES = 100; //number of lexical items to sample for likelihood approximation
const double GEOMETRIC = 1./2.; // geometric constant controling word length -- note that this is (1-g) in the usual sense!
const double CONS = 0.62; // probability of selecting a consonant (instead of a vowel)
const double PI = 3.1415926535897932384626433832795; // value of pi
const double LOGPI = 1.144729885849400174143; // value of log (pi)

// Type Definitions of Structures
typedef struct phone_struct {
	gsl_vector *formant_ptr; // vector corresponding to formant values of a particular acoustic token
	} phone;
typedef struct phoneme_struct {
	gsl_vector *mu_ptr; // vector corresponding to category mean
	gsl_matrix *varnu_ptr; // vector corresponding to category covariance
	double nu; // vector corresponding to category degrees of freedom (number of sounds assigned plus nu0)
	int nc; // number of times phoneme has been used, n_c
  int index; // index+1 of phoneme in the phoneme inventory, c_ind_ptr
	} phoneme;
typedef struct phoneme_slot_struct {
	phoneme *phon_ptr; // pointer to phonetic category
	gsl_vector *wkj_ptr; // mean of phonetic values assigned to that position of that lexical item
	gsl_matrix *ss_ptr; // sum of squares of phonetic values assigned to that position of that lexical item
	} phoneme_slot;
typedef struct lexical_item_struct {
	int *CV_ptr; // pointer to vector that indicates consonant or vowel
	int *cons_ptr; // pointer to vector of consonant identities
	phoneme_slot *vowel_ptr; // pointer to vector of vowel identities
	int lex_length; // length of lexical item
	int vowel_length; // number of vowels
	int cons_length; // number of consonants
	int nl; // number of times lexical item has been used, n_l
  int index; // index+1 of lexical item in the lexicon, l_ind_ptr
	} lexical_item;
typedef struct word_struct {
	int *CV_ptr; // vector that indicates consonant or vowel
	int *cons_ptr; // vector of consonant identities
	phone *vowel_ptr; // acoustic values for each vowel
	int word_length; // number of phonemes in word
	int vowel_length; // number of vowels in word
	int cons_length; // number of consonants in word
	lexical_item *z_ptr; // pointer to the lexical item to which it belongs
	} word;

// Global Variables
phoneme *c_ind_ptr[NUM_PHONES+1]; // restaurant of (pointers to) phonemes
lexical_item *l_ind_ptr[NUM_WORDS+1]; // restaurant of (pointers to) lexical items
word w[NUM_WORDS]; // list of words in the corpus
int ncons[NUM_CONSONANTS]; // keeps track of how many times consonants appeared
gsl_vector *mu0_ptr; // prior on mean
gsl_matrix *varnu0_ptr; // prior on variance
double nu0; // prior on degrees of freedom
phoneme *extra_phonemes_ptr[LONGEST]; // declare a vector of pointers to phonemes
int num_tables_c; // number of tables being used in phoneme inventory
int num_tables_l; // number of tables being used in lexicon
double alpha;
double beta;

// Function Declarations
void fileread (void);
void filewrite (int iteration, double ll);
void free_w (void);
double det (gsl_matrix *m_ptr, gsl_matrix *m_copy_ptr, gsl_permutation *p_ptr);
int random_sample (double probs[], double total_prob, int num_outcomes);
double t_likelihood (gsl_vector *wkj_ptr, gsl_matrix *ss_ptr, double n, gsl_vector *mu_ptr, double nu, gsl_matrix *varnu_ptr, gsl_matrix *variance_ptr, gsl_matrix *matrix1_ptr, gsl_vector *diff_ptr, gsl_permutation *p_ptr);
void add_values (gsl_vector *wkj_ptr, gsl_matrix *ss_ptr, double n, gsl_vector *mu_ptr, double *nu_ptr, gsl_matrix *varnu_ptr, gsl_vector *temp_ptr);
void subtract_values (gsl_vector *wkj_ptr, gsl_matrix *ss_ptr, double n, gsl_vector *mu_ptr, double *nu_ptr, gsl_matrix *varnu_ptr, gsl_vector *temp_ptr);
phoneme_slot *sample_lexical_item (double probs[], word *word_ptr, int n, double anneal);
lexical_item *create_lexical_item (word *word_ptr, phoneme_slot *vowel_ptr);
void destroy_lexical_item (lexical_item *lex_ptr);
phoneme *create_phoneme (void);
void destroy_phoneme (phoneme *phon_ptr);
void destroy_phoneme_index (phoneme *phon_ptr);
double joint_probability (gsl_matrix *matrix1_ptr, gsl_matrix *matrix2_ptr, gsl_vector *vector1_ptr, gsl_permutation *p_ptr);


/*******************************************************************************
void fileread(void) 

Reads values from a file.
********************************************************************************/

void fileread (void) 
{
  int i, j;
  FILE *lengths_file_ptr; // a file that contains lengths of each word in the corpus
  FILE *CV_file_ptr; // a file that contains syllable template information
  FILE *vowel_file_ptr; // a file that contains a list of acoustic values
  FILE *cons_file_ptr; // a file that contains a list of consonant identities

  // open the datafiles
  lengths_file_ptr = fopen("Simulation3-len.txt", "r");
  CV_file_ptr = fopen("Simulation3-CV.txt", "r");
  vowel_file_ptr = fopen("Simulation3-vowel.txt", "r");
  cons_file_ptr = fopen("Simulation3-cons.txt", "r");
  
  i = 0; // counter for which word we're reading in
  
  while (fscanf(lengths_file_ptr, "%i %i %i", &w[i].word_length, &w[i].cons_length, &w[i].vowel_length) != EOF) {
        
    w[i].CV_ptr = malloc (w[i].word_length * sizeof(int));
    if (w[i].CV_ptr == NULL)	{ fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
    for (j = 0; j < w[i].word_length; j++) {
      fscanf (CV_file_ptr, "%i", &w[i].CV_ptr[j]);
    }

    w[i].cons_ptr = malloc (w[i].cons_length * sizeof(int));
    if (w[i].cons_ptr == NULL)	{ fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
    for (j = 0; j < w[i].cons_length; j++) {
      fscanf (cons_file_ptr, "%i", &w[i].cons_ptr[j]);
    }

    w[i].vowel_ptr = malloc (w[i].vowel_length * sizeof(phone));
    if (w[i].vowel_ptr == NULL)	{ fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
    for (j = 0; j < w[i].vowel_length; j++) {
      w[i].vowel_ptr[j].formant_ptr = gsl_vector_alloc (DIM);
      gsl_vector_fscanf (vowel_file_ptr, w[i].vowel_ptr[j].formant_ptr);
    }
    
    i++;
    
    if (i == NUM_WORDS) {break;}

  }
  
  fclose (vowel_file_ptr);
  fclose (cons_file_ptr);
  fclose (CV_file_ptr);
  fclose (lengths_file_ptr);
}


/*******************************************************************************
void filewrite(int iteration, double ll) 

Writes results to a file.
- iteration is the iteration number, used in naming the files
********************************************************************************/

void filewrite (int iteration, double ll)
{
  FILE *z_file_ptr; // file with word assignments to lexical items
  FILE *l_file_ptr; // file with lexical assignments to phonemes
  FILE *c_file_ptr; // file with phoneme parameters
  FILE *ll_file_ptr; // file that has the joint probability
  
  char z[50], l[50], c[50], lik[50]; // sprintf needs variables to write to
  sprintf (z, "z_%i_%i_%i.txt", (int) (alpha), (int) (beta*10), iteration);
  sprintf (l, "l_%i_%i_%i.txt", (int) (alpha), (int) (beta*10), iteration);
  sprintf (c, "c_%i_%i_%i.txt", (int) (alpha), (int) (beta*10), iteration);
  sprintf (lik, "ll_%i_%i_%i.txt", (int) (alpha), (int) (beta*10), iteration);
  
  z_file_ptr = fopen (z, "w");
  l_file_ptr = fopen (l, "w");
  c_file_ptr = fopen (c, "w");
  ll_file_ptr = fopen (lik, "w");
  
  // print results to these files
  int i, j;
  // print word assignments vector z
  for (i = 0; i < NUM_WORDS; i++) {
    fprintf (z_file_ptr, "%i\n", w[i].z_ptr->index);
  }
  // print lexicon l
  for (i = 0; i < num_tables_l; i++) {
    for (j = 0; j < l_ind_ptr[i]->lex_length; j++) {
      fprintf (l_file_ptr, "%i ", l_ind_ptr[i]->CV_ptr[j]);
    }
    fprintf (l_file_ptr, "\n");
    for (j = 0; j < l_ind_ptr[i]->cons_length; j++) {
      fprintf (l_file_ptr, "%i ", l_ind_ptr[i]->cons_ptr[j]);
    }
    fprintf (l_file_ptr, "\n");
    for (j = 0; j < l_ind_ptr[i]->vowel_length; j++) {
      fprintf (l_file_ptr, "%i ", l_ind_ptr[i]->vowel_ptr[j].phon_ptr->index);
    }
    fprintf (l_file_ptr, "\n");
  }
  assert (l_ind_ptr [num_tables_l] == NULL);
  // print phonetic category parameters c
  for (i = 0; i < num_tables_c; i++) {
    fprintf (c_file_ptr, "%g %g\n%g %g %g %g\n%g\n", gsl_vector_get(c_ind_ptr[i]->mu_ptr,0), 
             gsl_vector_get(c_ind_ptr[i]->mu_ptr,1), gsl_matrix_get(c_ind_ptr[i]->varnu_ptr,0,0),
             gsl_matrix_get(c_ind_ptr[i]->varnu_ptr,0,1), gsl_matrix_get(c_ind_ptr[i]->varnu_ptr,1,0),
             gsl_matrix_get(c_ind_ptr[i]->varnu_ptr,1,1), c_ind_ptr[i]->nu);
  }
  fprintf (ll_file_ptr, "%g\n", ll);
  
  fclose (z_file_ptr);
  fclose (l_file_ptr);
  fclose (c_file_ptr);
  fclose (ll_file_ptr);
}

/*******************************************************************************
void free_w(void) 

Frees the memory associated with the corpus.
********************************************************************************/

void free_w (void) 
{
  int i, j;
  
  for (i = 0; i < NUM_WORDS; i++) {
    for (j = 0; j < w[i].vowel_length; j++) {
      // free memory associated with formant vectors
      gsl_vector_free (w[i].vowel_ptr[j].formant_ptr);
    }
    // free memory associated with different aspects of the word
    free (w[i].vowel_ptr);
    free (w[i].CV_ptr);
    free (w[i].cons_ptr);
  }
  
}


/*******************************************************************************
double det(gsl_matrix *m_ptr)

Returns the determinant of a matrix m_ptr, without overwriting the contents of m_ptr.
********************************************************************************/

double det (gsl_matrix *m_ptr, gsl_matrix *m_copy_ptr, gsl_permutation *p_ptr)
{
  int signum;
	gsl_matrix_memcpy (m_copy_ptr, m_ptr);
	gsl_linalg_LU_decomp (m_copy_ptr, p_ptr, &signum);  // LU decomposition
	double d = gsl_linalg_LU_det (m_copy_ptr, signum); // use the diagonal to compute the determinant
	return d;
}


/*******************************************************************************
int random_sample(probs, totprob, num_outcomes)

Returns a random sample from a discrete probability distribution.
- probs is a pointer to an array of counts (proportional to probabilities)
- totprob is the sum of everything in probs -- the proportionality constant
- num_outcomes is the number of discrete bins
********************************************************************************/

int random_sample (double probs[], double total_prob, int num_outcomes)
{
	int i = 0; // bin number
	double r = myrand() * total_prob; // pick a random number between 0 and totprob
	double cutoff = *probs; // maximum-number cutoff for r to fall into bin i
  //printf ("Number: %g; Totprob: %g\n", r, total_prob);
	while (r > cutoff) {
		i++;
		assert (i < num_outcomes); // make sure we haven't run out of options
		cutoff += *(probs+i); // pointer arithmetic
  }
  assert (probs[i] >= 0);
	return i; // returns a number between 0 and num_outcomes-1
}

/*******************************************************************************
double t_likelihood(wkj, ss, n, mu, nu, varnu)

Returns the log likelihood of a datapoint belonging to a category that contains 
a given set of other points.
The first 3 parameters describe the data:
- wkj is the mean of the query points
- ss is the sum of squares from the data, i.e. sum((WIJ-WKJ)^2)
- n is the number of query points
The last 3 parameters describe the category:
- mu is the mean of the likelihood
- nu is the degrees of freedom of the likelihood
- varnu is the variance * degrees of freedom of the likelihood
********************************************************************************/

double t_likelihood (gsl_vector *wkj_ptr, gsl_matrix *ss_ptr, double n, 
										 gsl_vector *mu_ptr, double nu, gsl_matrix *varnu_ptr, 
                     gsl_matrix *variance_ptr, gsl_matrix *matrix1_ptr, gsl_vector *diff_ptr, 
                     gsl_permutation *p_ptr)
{
	double p = 0; // log probability
	int i;
	gsl_matrix_memcpy (variance_ptr, varnu_ptr); // variance_ptr holds varnu
	gsl_matrix_add (variance_ptr, ss_ptr); // variance_ptr holds varnu+ss
	gsl_vector_memcpy (diff_ptr, wkj_ptr); // diff_ptr holds wkj
	gsl_vector_sub (diff_ptr, mu_ptr); // diff_ptr holds wkj-mu
	gsl_blas_dger ((n * nu)/(n + nu), diff_ptr, diff_ptr, variance_ptr);  // variance_ptr holds entire sums-of-squares term

	// compute the normalization constant
	for (i = 0; i < DIM; i++) {
		p += lgamma ((nu + n - (double) i) / 2.) - lgamma ((nu - (double) i) / 2.);
	}
	// compute the rest of the log probability
	p += (nu / 2) * log (det (varnu_ptr, matrix1_ptr, p_ptr)) - ((double) DIM * n/2) * LOGPI - ((double) DIM / 2) * log ((n + nu) / nu )
		- ((nu + n) / 2) * log (det (variance_ptr, matrix1_ptr, p_ptr));
		
	return p;
}


/*******************************************************************************
void add_values(wkj, ss, n, *mu, *nu, *varnu)

Adds the values from a set of datapoints to the parameters of a category.
The first 3 parameters describe the data:
- wkj is the mean of the query points
- ss is the sum of squares from the data, i.e. sum((WIJ-WKJ)^2)
- n is the number of query points
The last 3 parameters describe the category:
- mu is the mean of the likelihood
- nu is the degrees of freedom of the likelihood
- varnu is the variance * degrees of freedom of the likelihood
********************************************************************************/

void add_values (gsl_vector *wkj_ptr, gsl_matrix *ss_ptr, double n, 
							   gsl_vector *mu_ptr, double *nu_ptr, gsl_matrix *varnu_ptr, gsl_vector *temp_ptr)
{
	gsl_matrix_add(varnu_ptr, ss_ptr);
	gsl_vector_memcpy (temp_ptr, wkj_ptr); // temp_ptr holds wkj
	gsl_vector_sub (temp_ptr, mu_ptr); // temp_ptr holds wkj-mu
	gsl_blas_dger ((*nu_ptr * n) / (*nu_ptr + n), temp_ptr, temp_ptr, varnu_ptr);
	gsl_vector_scale (mu_ptr, (*nu_ptr / (*nu_ptr + n)));
	gsl_vector_memcpy (temp_ptr, wkj_ptr); // temp_ptr holds wkj
	gsl_vector_scale (temp_ptr,  n / (*nu_ptr + n) ); // temp_ptr holds a scaled version of wkj
	gsl_vector_add (mu_ptr, temp_ptr);
	*nu_ptr += n;
	// make sure the matrix is symmetric (this is for 2x2 covariance matrices -- need to generalize to more dimensions)
	if (gsl_matrix_get (varnu_ptr, 0, 1) != gsl_matrix_get (varnu_ptr, 1, 0)) {
		gsl_matrix_set (varnu_ptr, 0, 1, gsl_matrix_get (varnu_ptr, 1, 0));
	}
}

/*******************************************************************************
void subtract_values(wkj, ss, n, *mu, *nu, *varnu)

Subtracts the values from a set of datapoints to the parameters of a category.
The first 3 parameters describe the data:
- wkj is the mean of the query points
- ss is the sum of squares from the data, i.e. sum((WIJ-WKJ)^2)
- n is the number of query points
The last 3 parameters describe the category:
- mu is the mean of the likelihood
- nu is the degrees of freedom of the likelihood
- varnu is the variance * degrees of freedom of the likelihood
********************************************************************************/

void subtract_values (gsl_vector *wkj_ptr, gsl_matrix *ss_ptr, double n, 
                      gsl_vector *mu_ptr, double *nu_ptr, gsl_matrix *varnu_ptr, gsl_vector *temp_ptr)
{
	*nu_ptr -= n;
	gsl_vector_memcpy (temp_ptr, wkj_ptr); // temp_ptr holds wkj
  gsl_vector_scale (temp_ptr,  n / (*nu_ptr + n) ); // temp_ptr holds a scaled version of wkj
  gsl_vector_sub (mu_ptr, temp_ptr); // subtract scaled wkj
  gsl_vector_scale (mu_ptr, (*nu_ptr + n) / *nu_ptr ); // multiply by appropriate constant
	gsl_vector_memcpy (temp_ptr, wkj_ptr); // temp_ptr holds wkj
	gsl_vector_sub (temp_ptr, mu_ptr); // temp_ptr holds wkj-mu
  gsl_matrix_sub (varnu_ptr, ss_ptr);
  gsl_blas_dger (-1. * (*nu_ptr * n) / (*nu_ptr + n), temp_ptr, temp_ptr, varnu_ptr);

	// make sure the matrix is symmetric (this is for 2x2 covariance matrices -- need to generalize to more dimensions)
	if (gsl_matrix_get (varnu_ptr, 0, 1) != gsl_matrix_get (varnu_ptr, 1, 0)) {
		gsl_matrix_set (varnu_ptr, 0, 1, gsl_matrix_get (varnu_ptr, 1, 0));
	}
}

/*******************************************************************************
lexical_item *sample_lexical_item (*probs, totprob, word_length, n)

Samples an array of phonemes from the prior.
- probs[] is an array of phoneme counts, proportional to probabilities
- totprob is the total of everything in probs
- word_ptr gives the number of phonemes to sample
- n is the number of occupied tables plus one unoccupied table
********************************************************************************/

phoneme_slot *sample_lexical_item (double probs[], word *word_ptr, int n, double anneal)
{
	// copy probabilities to new table so the old one isn't modified
  double probs_copy_orig[n+word_ptr->vowel_length];
	double probs_copy[n+word_ptr->vowel_length];
	int i;
  // compute probabilities raised to an annealing power
  double *max_prob_ptr = probs_copy; // pointer to highest probability
	for (i = 0; i < n; i++) {
    probs_copy_orig[i] = probs[i]; // save for later
    probs_copy[i] = log(probs[i]);
    if (probs_copy[i] > *max_prob_ptr) {
      max_prob_ptr = probs_copy+i;
    }
	}
  for (i = 0; i < n; i++) {
    probs_copy[i] = log(probs_copy_orig[i]);
    if (probs_copy[i] > *max_prob_ptr) {
      max_prob_ptr = probs_copy+i;
    }
  }
  double max = *max_prob_ptr;
  max_prob_ptr = NULL;
  double total_prob = 0.;
  for (i = 0; i < n; i++) {
    probs_copy[i] = exp (anneal * (probs_copy[i]-max));
    total_prob += probs_copy[i];
  }
  
  // initialize the extra phonemes
  for (i = n; i < n+word_ptr->vowel_length; i++) {
    probs_copy_orig[i] = 0.;
    probs_copy[i] = 0.;
  }


	int n_copy = n; // the number of phonemes represented in probs_copy_orig
	// initialize a new array of phonemes
	phoneme_slot *lex_ptr = malloc (word_ptr->vowel_length * sizeof(phoneme_slot));
	if (lex_ptr == NULL)	{ fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
	// sample assignments for the vowels
	for (i = 0; i < word_ptr->vowel_length; i++) {
    // sample a phoneme
    lex_ptr[i].phon_ptr = NULL; // for bookkeeping, and to avoid errors
		int r = random_sample(probs_copy, total_prob, n_copy); // pick a random number between 0 and n-1
		if (r == n-1) { // if it's a brand new phoneme
			lex_ptr[i].phon_ptr = extra_phonemes_ptr[i];
      probs_copy_orig[n_copy] = 1.; // add a count to the last possible phoneme
      probs_copy[n_copy] = exp (anneal * (0.-max));
      total_prob += probs_copy[n_copy];
		} else if (r > n-1) { // if it's a NEW phoneme that's already been used in this word
			// copy the phoneme from that slot, e.g. r==n copies from slot 0
			lex_ptr[i].phon_ptr = lex_ptr[r-n].phon_ptr;
      // add a count to the probability of the phoneme in that slot
      total_prob -= probs_copy[r-1];
      probs_copy_orig[r-1]++;
      probs_copy[r-1] = exp (anneal * (log(probs_copy_orig[r-1])-max));
      total_prob += probs_copy[r-1];
		} else if (r < n-1) {
			lex_ptr[i].phon_ptr = c_ind_ptr[r]; // copy the pointer from the phoneme list
      total_prob -= probs_copy[r];
      probs_copy_orig[r]++;
      probs_copy[r] = exp (anneal * (log(probs_copy_orig[r])-max));
      total_prob += probs_copy[r];
		}
    assert (lex_ptr[i].phon_ptr != NULL);
		// add a count to a new category corresponding to the phoneme that occupies this slot in the word
		n_copy++; // allow more categories on the next time through
	}
	return lex_ptr; // return the pointer to the new lexical item
}


/*******************************************************************************
lexical_item *create_lexical_item (word *word_ptr, phoneme_slot *initialize_ptr)

Creates a lexical item based on a word template and a string of vowels
- word_ptr is a word on which this lexical item's syllable template and consonants are based
- initialize_ptr is the string of vowels that has already been initialized or sampled
********************************************************************************/

lexical_item *create_lexical_item (word *word_ptr, phoneme_slot *initialize_ptr)
{
	// allocate enough memory for a new lexical item
	lexical_item *lex_ptr = malloc(sizeof(lexical_item)); 	
	if (lex_ptr == NULL) { fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
	// allocate space for the syllable template
	lex_ptr->CV_ptr = malloc(word_ptr->word_length * sizeof(int));
	if (lex_ptr->CV_ptr == NULL)	{ fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
  int i;
	for (i = 0; i < word_ptr->word_length; i++) {
		lex_ptr->CV_ptr[i] = word_ptr->CV_ptr[i]; // copy the syllable template of the word
	}
	// allocate space for each consonant in the lexical item
	lex_ptr->cons_ptr = malloc(word_ptr->cons_length * sizeof(int));
	if (lex_ptr->cons_ptr == NULL)	{ fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
	for (i = 0; i < word_ptr->cons_length; i++) {
		lex_ptr->cons_ptr[i] = word_ptr->cons_ptr[i]; // copy the consonants of the word
    ncons[lex_ptr->cons_ptr[i]-1]++; // increment the consonant counts
	}
	// set the pointer to the string of vowels
	lex_ptr->vowel_ptr = initialize_ptr;
  for (i = 0; i < word_ptr->vowel_length; i++) {
    // if it's a new phoneme, add to the phoneme inventory
	  if (lex_ptr->vowel_ptr[i].phon_ptr->nc == 0) { // this means it's a new phoneme
      //printf ("Creating phoneme %i\n", num_tables_c+1);
		  // copy the pointer to the restaurant of phonemes
		  c_ind_ptr[num_tables_c] = lex_ptr->vowel_ptr[i].phon_ptr;
      // set the index value of this phoneme
      c_ind_ptr[num_tables_c]->index = num_tables_c+1;
	  	// increase the number of tables
	  	num_tables_c++;
	  	// and set the null value marker for the first empty table
	  	c_ind_ptr[num_tables_c] = NULL;
      // create a new phoneme in the bank of new phonemes, so this one doesn't get reused
      extra_phonemes_ptr[i] = create_phoneme ();
	  }
    // allocate space for parameters of vowel, initialize to zero
		lex_ptr->vowel_ptr[i].wkj_ptr = gsl_vector_calloc (DIM);
		lex_ptr->vowel_ptr[i].ss_ptr = gsl_matrix_calloc (DIM, DIM);
    // increment the vowel counts
    lex_ptr->vowel_ptr[i].phon_ptr->nc++;
  }
	// initialize all the other variables based on the word
	lex_ptr->lex_length = word_ptr->word_length;
	lex_ptr->vowel_length = word_ptr->vowel_length;
	lex_ptr->cons_length = word_ptr->cons_length;
	lex_ptr->nl = 0;
	return lex_ptr; // return the pointer to the new lexical item
}

/*******************************************************************************
void destroy_lexical_item(*lex_ptr)

Frees the memory associated with a lexical item.
- lex_ptr is a pointer to the lexical item that needs to be freed
********************************************************************************/

void destroy_lexical_item (lexical_item *lex_ptr)
{
	int i, j;
	for (i = 0; i < lex_ptr->vowel_length; i++) {
		gsl_vector_free (lex_ptr->vowel_ptr[i].wkj_ptr);
    lex_ptr->vowel_ptr[i].wkj_ptr = NULL;
		gsl_matrix_free (lex_ptr->vowel_ptr[i].ss_ptr);
    lex_ptr->vowel_ptr[i].ss_ptr = NULL;
		lex_ptr->vowel_ptr[i].phon_ptr->nc--;
		if (lex_ptr->vowel_ptr[i].phon_ptr->nc == 0) {
      //printf ("Destroying phoneme %i\n", lex_ptr->vowel_ptr[i].phon_ptr->index);
			// destroy one of the extra phonemes
			destroy_phoneme (extra_phonemes_ptr[i]);
			// and replace it with the phoneme from this word
			extra_phonemes_ptr[i] = lex_ptr->vowel_ptr[i].phon_ptr;
			// then remove the table from the phoneme restaurant
			destroy_phoneme_index (lex_ptr->vowel_ptr[i].phon_ptr);
			num_tables_c--;
		}
	}
  // don't free the memory associated with the vowel vector because it's being used as a sample from the prior
  for (i = 0; i < lex_ptr->cons_length; i++) {
    ncons[lex_ptr->cons_ptr[i]-1]--; // decrement the consonant counts
  }
  free (lex_ptr->cons_ptr); // free the consonant vector
  lex_ptr->cons_ptr = NULL;
	free (lex_ptr->CV_ptr); // free the syllable template vector
  lex_ptr->CV_ptr = NULL;
	// find the table number to which this lexical item corresponds
	i = 0;
	while (l_ind_ptr[i] != lex_ptr) {
		i++;
		assert (l_ind_ptr[i] != NULL);
	}
  for (j = i; l_ind_ptr[j] != NULL; j++) { // there will be a null value marking the end of the restaurant
		l_ind_ptr[j] = l_ind_ptr[j+1]; // transpose the table identities by one
    if (l_ind_ptr[j] != NULL) {
      l_ind_ptr[j]->index--; // decrease the index value of the lexical item
      assert (l_ind_ptr[j]->index > 0);
    }
	}
	free (lex_ptr);
}

/*******************************************************************************
phoneme *create_phoneme(void)

Allocates memory for a new phoneme, and initializes its values.
********************************************************************************/

phoneme *create_phoneme (void)
{
	phoneme *phon_ptr = malloc(sizeof(phoneme));
	if (phon_ptr == NULL)	{ fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
	phon_ptr -> mu_ptr = gsl_vector_alloc (DIM);
  gsl_vector_memcpy (phon_ptr -> mu_ptr, mu0_ptr);
	phon_ptr -> varnu_ptr = gsl_matrix_alloc (DIM, DIM);
  gsl_matrix_memcpy (phon_ptr -> varnu_ptr, varnu0_ptr);
	phon_ptr -> nu = nu0;
	phon_ptr -> nc = 0;
  phon_ptr -> index = 0; // for bookkeeping
	return phon_ptr;
}

/*******************************************************************************
void destroy_phoneme(*phon_ptr)

Frees the memory associated with a phoneme.
- phon_ptr is a pointer to the phoneme that needs to be freed
********************************************************************************/

void destroy_phoneme (phoneme *phon_ptr)
{
  gsl_vector_free (phon_ptr -> mu_ptr);
  phon_ptr -> mu_ptr = NULL;
  gsl_matrix_free (phon_ptr -> varnu_ptr);
  phon_ptr -> varnu_ptr = NULL;
	free (phon_ptr);
}


/*******************************************************************************
void destroy_phoneme_index(*phon_ptr)

Deletes a phoneme from the phoneme inventory, but doesn't free the memory.
- phon_ptr is a pointer to the phoneme whose index needs to be deleted
********************************************************************************/

void destroy_phoneme_index (phoneme *phon_ptr)
{
	// find the table number to which this phoneme corresponds
	int i = 0, j;
	while (c_ind_ptr[i] != phon_ptr) {
		i++;
		assert (c_ind_ptr[i] != NULL);
	}
	for (j = i; c_ind_ptr[j] != NULL; j++) { // null value marks the end of the restaurant
		c_ind_ptr[j] = c_ind_ptr[j+1]; // transpose the table identities by one
    if (c_ind_ptr[j] != NULL) {
      c_ind_ptr[j]->index--; // decrease index number on that phoneme
      assert (c_ind_ptr[j]->index > 0);
    }
	}
}


/*******************************************************************************
double joint_probability(void)

Computes the joint probability of the data and category assignments by factorizing
the joint probability of the data:
p(w,z,l|alpha,beta,mu0,nu0,var0) = p(w|l,z,mu0,nu0,var0) p(z|l,alpha) p(l|beta)
********************************************************************************/

double joint_probability (gsl_matrix *matrix1_ptr, gsl_matrix *matrix2_ptr, gsl_vector *vector1_ptr, gsl_permutation *p_ptr)
{
  // p(l|beta)
  // The numerator is beta * 1 * 2 * ... * Nc-1 for each category
  // The denominator is beta * (1+beta) * (2+beta) * ... * (NC-1+beta)
  // Also incorporate the probability of a particular word length and syllable template
  int i, j, k;
  double total_n = 0;
  double logprob = ((double) num_tables_c + (double) NUM_CONSONANTS) * log (beta); // beta factor in the numerator
  // compute the rest of the numerator for vowels
  for (i = 0; i < num_tables_c; i++) {
    logprob += lgamma ((double) c_ind_ptr[i]->nc);
    total_n += (double) c_ind_ptr[i]->nc;
  }
  // compute the denominator for vowels
  for (i = 0; i < total_n; i++) {
    logprob -= log ((double) i + beta);
  }
  // compute the rest of the numerator for consonants
  total_n = 0;
  for (i = 0; i < NUM_CONSONANTS; i++) {
    logprob += lgamma ((double) ncons[i]);
    total_n += (double) ncons[i];
  }
  // compute the denominator for consonants
  for (i = 0; i < total_n; i++) {
    logprob -= log ((double) i + beta);
  }
  // compute the probability of the syllable template
  for (i = 0; i < num_tables_l; i++) {
    // geometric distribution on lengths
    logprob += ((double) l_ind_ptr[i]->lex_length-1.) * log (GEOMETRIC) + log (1-GEOMETRIC);
    // probability of choosing either C or V for each slot
    logprob += (double) l_ind_ptr[i]->cons_length * log (CONS) + (double) l_ind_ptr[i]->vowel_length * log (1-CONS);
  }
  
  // p(z|l,alpha)
  // The numerator is alpha * 1 * 2 * ... * Nl-1 for each category
  // The denominator is alpha * (1+alpha) * (2+alpha) * ... * (NL-1+alpha)
  total_n = 0.;
  logprob += (double) num_tables_l * log (alpha); // alpha factor in the numerator
  // compute the rest of the numerator
  for (i = 0; i < num_tables_l; i++) {
    logprob += lgamma ((double) l_ind_ptr[i]->nl);
    total_n += (double) l_ind_ptr[i]->nl;
  }
  // compute the denominator
  for (i = 0; i < total_n; i++) {
    logprob -= log ((double) i + alpha);
  }
  
  // p(w|l,z,mu0,nu0,var0)
  // Probability of the acoustic values given their phoneme assignments
  gsl_vector *vzeros_ptr = gsl_vector_calloc (DIM);
  gsl_matrix *mzeros_ptr = gsl_matrix_calloc (DIM, DIM);
  for (i = 0; i < num_tables_l; i++) {
    for (j = 0; j < l_ind_ptr[i]->vowel_length; j++) {
      // reset the mean and sum of squares for each position in each lexical item
      gsl_vector_memcpy (l_ind_ptr[i]->vowel_ptr[j].wkj_ptr, vzeros_ptr);
      gsl_matrix_memcpy (l_ind_ptr[i]->vowel_ptr[j].ss_ptr, mzeros_ptr);
    }
  }
  gsl_vector *wkj_ptr = gsl_vector_alloc (DIM);
  gsl_matrix *ss_ptr = gsl_matrix_alloc (DIM, DIM);
  for (i = 0; i < num_tables_c; i++) { // loop through all the phonemes
    // Reset the mean, covariance, and df of the phoneme
    gsl_vector_memcpy (c_ind_ptr[i]->mu_ptr, mu0_ptr);
    gsl_matrix_memcpy (c_ind_ptr[i]->varnu_ptr, varnu0_ptr);
    c_ind_ptr[i]->nu = nu0;
    int n = 0; // count the number of acoustic values assigned to the phoneme
    // Initialize mean and ss variables to zero
    gsl_vector_memcpy (wkj_ptr, vzeros_ptr);
    gsl_matrix_memcpy (ss_ptr, mzeros_ptr);
    // Find instances of that phoneme
    for (j = 0; j < NUM_WORDS; j++) { // search through all words
      for (k = 0; k < w[j].vowel_length; k++) { // through all positions in each word
        if (w[j].z_ptr->vowel_ptr[k].phon_ptr == c_ind_ptr[i]) {
          // add the value to the mean/ss for the corresponding lexical item
          gsl_vector_add (w[j].z_ptr->vowel_ptr[k].wkj_ptr, w[j].vowel_ptr[k].formant_ptr); // add the formant values for the phone
          gsl_blas_dger (1., w[j].vowel_ptr[k].formant_ptr, w[j].vowel_ptr[k].formant_ptr, w[j].z_ptr->vowel_ptr[k].ss_ptr); // calculate sum of squares
          // add the value to the running mean/cov/df counts for the phoneme
          gsl_vector_add (wkj_ptr, w[j].vowel_ptr[k].formant_ptr);
          gsl_blas_dger (1., w[j].vowel_ptr[k].formant_ptr, w[j].vowel_ptr[k].formant_ptr, ss_ptr);
          n++;
        }
      }
    }
    // Correct the mean and sum-of-squares terms
    gsl_vector_scale (wkj_ptr, 1./(double) n); // normalize the mean, E[X]
    gsl_blas_dger (-1.*(double) n, wkj_ptr, wkj_ptr, ss_ptr); // n*E[X^2]-n*E[X]^2
    // Multiply by the likelihood of these points
    logprob += t_likelihood (wkj_ptr, ss_ptr, (double) n, c_ind_ptr[i]->mu_ptr, c_ind_ptr[i]->nu, c_ind_ptr[i]->varnu_ptr, 
                             matrix1_ptr, matrix2_ptr, vector1_ptr, p_ptr);
    // Compute the new mean and covariance counts for the phoneme
    add_values (wkj_ptr, ss_ptr, (double) n, c_ind_ptr[i]->mu_ptr, &c_ind_ptr[i]->nu, c_ind_ptr[i]->varnu_ptr, vector1_ptr);
  }
  
  
  gsl_vector_free (wkj_ptr);
  gsl_matrix_free (ss_ptr);
  
  gsl_vector_free (vzeros_ptr);
  gsl_matrix_free (mzeros_ptr);
  
  return logprob;

}

/*******************************************************************************
int main()

This main program has three stages associated with it:
(1) Initialization
		- Set priors on phonetic category parameters
		- Initialize all words to different lexical items
		- Initialize all phoneme slots in all lexical items to the same phoneme
(2) Re-sample L (phoneme slot assignments)
		- Compute likelihood for all phonemes
		- Sample a phoneme
(3) Re-sample Z (word assignments)
		- Generate a bunch of lexical items from the prior
		- Determine which current lexical items are consistent with consonant template
		- Compute likelihood for all consistent lexical items
		- Sample a lexical item
********************************************************************************/


int main()
{

  seedMT (time(NULL));
//	unsigned int iseed = (unsigned int)time(NULL);
//  srand (iseed);
  
	int i, j, k;
  
  // To print the first few random numbers
  for (i=0; i<0; i++) {
    double r = myrand();
    printf ("%g\n",r);
  }
  
	// Read data from a file
	printf ("Reading file...\n");
  fileread();
  
	// Local Variables
	phoneme_slot *pseudo_word[NUM_LEXICAL_SAMPLES]; // for lexical items sampled from the prior
  gsl_matrix *zeros = gsl_matrix_calloc (DIM,DIM); // zero covariance matrix for bookkeeping
  
  // Bookkeeping matrices that get passed to functions and overwritten each time
  gsl_matrix *matrix1_ptr = gsl_matrix_alloc (DIM, DIM);
  gsl_matrix *matrix2_ptr = gsl_matrix_alloc (DIM, DIM);
  gsl_vector *vector1_ptr = gsl_vector_alloc (DIM);
  gsl_permutation *p_ptr = gsl_permutation_alloc (DIM);
  
	/* INITIALIZE */
  
  printf ("Setting constants...\n");
	/* Priors on phonetic category parameters, a.k.a. Even More Constants */
	mu0_ptr = gsl_vector_alloc (DIM); // prior on mean
	gsl_vector_set (mu0_ptr, 0, 500.); // set prior to the correct value
	gsl_vector_set (mu0_ptr, 1, 1500.);
	varnu0_ptr = gsl_matrix_alloc (DIM,DIM); // prior on variance
	gsl_matrix_set_identity (varnu0_ptr); // set to the identity matrix
  nu0 = (double) DIM - 1. + .001; // prior on degrees of freedom
  /* Done with prior on phonetic category parameters */
	
  printf ("Initializing extra phonemes...\n");
	// Initialize a small restaurant of potential new phonemes
	for (i = 0; i < LONGEST; i++) {
		extra_phonemes_ptr[i] = create_phoneme();
	}


  for (alpha = 10000; alpha < 100000; alpha *= 10) {
    for (beta = 10; beta < 100; beta *= 10) {

      /* INITIALIZATION */
      gsl_vector *wkj_ptr = gsl_vector_calloc (DIM); // initialize wkj, set elements to zero
      gsl_matrix *ss_ptr = gsl_matrix_calloc (DIM, DIM); // initialize ss
      num_tables_c = 0;
      num_tables_l = 0;

    /*  
    // INITIALIZATION OPTION 1: DIFFERENT LEXICAL ITEMS, COMPOSED OF A SINGLE PHONEME

      phoneme *init_phoneme_ptr = extra_phonemes_ptr[0]; // the one phoneme that contains all phones
      for (i = 0; i < 2; i++) {
        // Allocate space for vowels in each lexical item, creating a new lexical item for each word
        phoneme_slot *initialization_ptr = malloc (w[i].vowel_length * sizeof(phoneme_slot));
        if (initialization_ptr == NULL) { fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
        // Initially, assign all vowels to the same phoneme
        for (j = 0; j < w[i].vowel_length; j++) {
          initialization_ptr[j].phon_ptr = init_phoneme_ptr;
        }
        // Initialize the rest of the parameters in the lexical item
        l_ind_ptr[i] = create_lexical_item (w+i, initialization_ptr); // w+i is pointer-arithmetic for element i of array w
        l_ind_ptr[i]->nl = 1; // set the count associated with that lexical item
        l_ind_ptr[i]->index = i+1; // set the index value of this lexical item
        num_tables_l++; // increment the number of lexical items
        w[i].z_ptr = l_ind_ptr[i]; // assign this lexical identity to the word
        for (j = 0; j < w[i].vowel_length; j++) {
          // Set parameters associated with that vowel in the lexical item
          gsl_vector_memcpy (l_ind_ptr[i]->vowel_ptr[j].wkj_ptr,  w[i].vowel_ptr[j].formant_ptr); // set the acoustic values associated with the lexical item
          gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, l_ind_ptr[i]->vowel_ptr[j].ss_ptr); // calculate sum of squares
          // Update the acoustic parameters associated with the phoneme
          gsl_vector_add (wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // add the formant values for the phone
          gsl_matrix_add (ss_ptr, l_ind_ptr[i]->vowel_ptr[j].ss_ptr); // add the ss value stored with the lexical item
        }
      }

      for (i = 2; i < NUM_WORDS; i++) {
        w[i].z_ptr = l_ind_ptr[(i%2)];
        // Update the parameters in the lexical item
        w[i].z_ptr->nl++; // set the count associated with that lexical item
        for (j = 0; j < w[i].vowel_length; j++) {
          // Set parameters associated with that vowel in the lexical item
          gsl_vector_add (w[i].z_ptr->vowel_ptr[j].wkj_ptr,  w[i].vowel_ptr[j].formant_ptr); // set the acoustic values associated with the lexical item
          gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, w[i].z_ptr->vowel_ptr[j].ss_ptr); // calculate sum of squares
          // Update the acoustic parameters associated with the phoneme
          gsl_vector_add (wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // add the formant values for the phone
          gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, ss_ptr); // calculate sum of squares
        }
      }    

      for (i = 0; i < NUM_WORDS; i++) {
        // Allocate space for vowels in each lexical item, creating a new lexical item for each word
        phoneme_slot *initialization_ptr = malloc (w[i].vowel_length * sizeof(phoneme_slot));
        if (initialization_ptr == NULL) { fprintf(stderr, "out of memory\n"); exit(EXIT_FAILURE); }
        // Initially, assign all vowels to the same phoneme
        for (j = 0; j < w[i].vowel_length; j++) {
          initialization_ptr[j].phon_ptr = init_phoneme_ptr;
        }
        // Initialize the rest of the parameters in the lexical item
        l_ind_ptr[i] = create_lexical_item (w+i, initialization_ptr); // w+i is pointer-arithmetic for element i of array w
        l_ind_ptr[i]->nl = 1; // set the count associated with that lexical item
        l_ind_ptr[i]->index = i+1; // set the index value of this lexical item
        num_tables_l++; // increment the number of lexical items
        w[i].z_ptr = l_ind_ptr[i]; // assign this lexical identity to the word
        for (j = 0; j < w[i].vowel_length; j++) {
          // Set parameters associated with that vowel in the lexical item
          gsl_vector_memcpy (l_ind_ptr[i]->vowel_ptr[j].wkj_ptr,  w[i].vowel_ptr[j].formant_ptr); // set the acoustic values associated with the lexical item
          gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, l_ind_ptr[i]->vowel_ptr[j].ss_ptr); // calculate sum of squares
          // Update the acoustic parameters associated with the phoneme
          gsl_vector_add (wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // add the formant values for the phone
          gsl_matrix_add (ss_ptr, l_ind_ptr[i]->vowel_ptr[j].ss_ptr); // add the ss value stored with the lexical item
        }
      }
      
      // Correct the mean and sum-of-squares terms
      gsl_vector_scale (wkj_ptr, 1./ (double) NUM_PHONES); // normalize the mean, E[X]; NUM_PHONES is the number of acoustic values incorporated
      gsl_blas_dger (-1.*(double) NUM_PHONES, wkj_ptr, wkj_ptr, ss_ptr); // n*E[X^2]-n*E[X]^2

      // Update the acoustic parameters associated with the phoneme
      add_values (wkj_ptr, ss_ptr, (double) NUM_PHONES, c_ind_ptr[0]->mu_ptr, &c_ind_ptr[0]->nu, c_ind_ptr[0]->varnu_ptr, vector1_ptr);

      assert (num_tables_c == 1); // number of tables being used in phoneme inventory
      assert (num_tables_l == NUM_WORDS); // number of tables being used in lexicon
      // Make sure the ends of the arrays are set
      l_ind_ptr[NUM_WORDS] = NULL;
      c_ind_ptr[1] = NULL;

      */


      // INITIALIZATION OPTION 2: START BY SAMPLING LEXICAL ITEMS
      
      double anneal = (log (2.)) / ANNEALING_CONSTANT;

      for (i = 0; i < NUM_CONSONANTS; i++) {
        ncons[i] = 0;
      }
      
      // loop through all words in the corpus
      for (i = 0; i < NUM_WORDS; i++) {
        // Sample lexical items
        double probs_c[num_tables_c+1];
        for (j = 0; j < num_tables_c; j++) {
          probs_c[j] = (double) c_ind_ptr[j]->nc;
        }
        probs_c[num_tables_c] = beta;
        for (j = 0; j < NUM_LEXICAL_SAMPLES; j++) {
          pseudo_word[j] = sample_lexical_item(probs_c, w+i, num_tables_c+1, anneal);
        }
                
        // Figure out the effective value of alpha for this lexical frame
        // Divide by the number of samples from the prior
        double log_eff_alpha = log (alpha) - log ((double) NUM_LEXICAL_SAMPLES);
        // Term for length: g^(length-1) * (1-g)
        log_eff_alpha += ((double) w[i].word_length-1.) * log (GEOMETRIC);
        log_eff_alpha += log (1.-GEOMETRIC);
        // Term for syllable template: p_cons^num_cons * (1-p_cons)^num_vowels
        log_eff_alpha += (double) w[i].cons_length * log (CONS);
        log_eff_alpha += (double) w[i].vowel_length * log (1-CONS);
        // Term for particular consonants
        int my_ncons[NUM_CONSONANTS], total_ncons = 0;
        for (j = 0; j < NUM_CONSONANTS; j++) {
          my_ncons[j] = 0; // initialize the array
          total_ncons += ncons[j]; // meanwhile, compute the total number of consonants in the lexicon
        }
        for (j = 0; j < w[i].cons_length; j++) {
          my_ncons[w[i].cons_ptr[j]-1]++; // consonants numbered from 1, but array indexed from 0
          // divide by the denominator total_ncons * total_ncons+1 * ... * total_ncons+cons_length-1
          if (((double) total_ncons + (double) j) > 0. ) {
            log_eff_alpha -= log ((double) total_ncons + (double) j);
          } else {
            log_eff_alpha -= log (beta);
          }
        }
        for (j = 0; j < NUM_CONSONANTS; j++) {
          // numerator is ncons[j] * ncons[j]+1 * ... * ncons[j]+my_ncons[j]-1
          if (my_ncons[j] > 0) {
            if (ncons[j] > 0) {
              log_eff_alpha += log ((double) ncons[j]);
            } else {
              log_eff_alpha += log (beta);
            }
            // will only go through this loop if my_ncons[j]>1
            for (k = 1; k < my_ncons[j]; k++) {
              log_eff_alpha += log ((double) ncons[j]+(double) k);
            }
          }
        }
        
        // Figure out which lexical items are candidates for this word
        int cand[num_tables_l];
        for (k = 0; k < num_tables_l; k++) {
          // Initially assume it is a candidate
          cand[k] = 1;
          // Check whether the word is the right length
          if (w[i].word_length != l_ind_ptr[k]->lex_length) { 
            cand[k] = 0;
          } else {
            // If the right length, check the syllable template
            for (j = 0; j < l_ind_ptr[k]->lex_length; j++) {
              if (w[i].CV_ptr[j] != l_ind_ptr[k]->CV_ptr[j]) { 
                cand[k] = 0;
                break; // break out of the for loop, don't check any more CV slots
              }
            }
          }
          // If it's still a candidate, check the consonant identities
          if (cand[k] == 1) {
            for (j = 0; j < l_ind_ptr[k]->cons_length; j++) {
              if (w[i].cons_ptr[j] != l_ind_ptr[k]->cons_ptr[j]) {
                cand[k] = 0;
                break; // break out of the for loop, don't check any more consonant slots
              }
            }
          }
        }
        
        // Construct the table of probabilities
        double probs_l[num_tables_l+NUM_LEXICAL_SAMPLES];
        double *max_prob_ptr = probs_l+num_tables_l; // keep track of the highest probability, to subtract it off
        
        // Indicator vector to say whether phonemes are duplicated within a lexical item
        int duplicate[w[i].vowel_length]; 
        
        //printf ("Likelihood of newly sampled tables...\n");
        // Compute the log likelihood of newly sampled tables
        for (k = 0; k < NUM_LEXICAL_SAMPLES; k++) {
          probs_l[num_tables_l+k] = 0.;
          // Initialize the duplicate-indicator vector
          for (j = 0; j < w[i].vowel_length; j++) {
            duplicate[j] = 0;
          }
          for (j = 0; j < w[i].vowel_length; j++) {
            if (duplicate[j] == 0) { // if it's not a duplicate from before
              // Search through to see whether there are duplicate phonemes in the word
              gsl_vector_memcpy (wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // initialize wkj to formant value
              gsl_matrix_memcpy (ss_ptr, zeros);
              gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, ss_ptr); // initialize ss term
              int n = 1; // number of times phoneme occurs in the word
              int l;
              for (l = j+1; l < w[i].vowel_length; l++) {
                if (pseudo_word[k][j].phon_ptr == pseudo_word[k][l].phon_ptr) {
                  duplicate[l] = 1;
                  // Combine the parameters of the duplicates
                  gsl_vector_add (wkj_ptr, w[i].vowel_ptr[l].formant_ptr);
                  gsl_blas_dger (1., w[i].vowel_ptr[l].formant_ptr, w[i].vowel_ptr[l].formant_ptr, ss_ptr);
                  n++;
                }
              }
              gsl_vector_scale (wkj_ptr, 1./ (double) n); // normalize the mean, E[X]
              gsl_blas_dger (-1.*(double) n, wkj_ptr, wkj_ptr, ss_ptr); // n*E[X^2]-n*E[X]^2
              // compute the likelihood of the acoustic values for that phoneme
              probs_l[num_tables_l+k] += t_likelihood (wkj_ptr, ss_ptr, (double) n,
                pseudo_word[k][j].phon_ptr->mu_ptr, pseudo_word[k][j].phon_ptr->nu,
                pseudo_word[k][j].phon_ptr->varnu_ptr, matrix1_ptr, matrix2_ptr, vector1_ptr, p_ptr);
            }
          }
          probs_l[num_tables_l+k] += log_eff_alpha; // multiply by something proportional to the prior
          // keep a running count of the maximum, to subtract off before exponentiating
          if (probs_l[num_tables_l+k] > *max_prob_ptr) {
            max_prob_ptr = probs_l+num_tables_l+k; // pointer arithmetic for this log probability
          }
        }
        
        // Compute the log likelihood of existing tables
        for (k = 0; k < num_tables_l; k++) {
          if (cand[k] == 1) { // cand[k] will be 1 for candidates, 0 otherwise
            probs_l[k] = 0.;
            // Initialize the duplicate-indicator vector
            for (j = 0; j < w[i].vowel_length; j++) {
              duplicate[j] = 0;
            }
            for (j = 0; j < w[i].vowel_length; j++) {
              if (duplicate[j] == 0) { // if it's not a duplicate from before
                // Search through to see whether there are duplicate phonemes in the word
                gsl_vector_memcpy (wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // initialize wkj to formant value
                gsl_matrix_memcpy (ss_ptr, zeros);
                gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, ss_ptr); // initialize ss term
                int n = 1; // number of times phoneme occurs in the word
                int l;
                for (l = j+1; l < w[i].vowel_length; l++) {
                  if (l_ind_ptr[k]->vowel_ptr[j].phon_ptr == l_ind_ptr[k]->vowel_ptr[l].phon_ptr) {
                    duplicate[l] = 1;
                    // Combine the parameters of the duplicates
                    gsl_vector_add (wkj_ptr, w[i].vowel_ptr[l].formant_ptr);
                    gsl_blas_dger (1., w[i].vowel_ptr[l].formant_ptr, w[i].vowel_ptr[l].formant_ptr, ss_ptr);
                    n++;
                  }
                }
                gsl_vector_scale (wkj_ptr, 1./ (double) n); // normalize the mean, E[X]
                gsl_blas_dger (-1.*(double) n, wkj_ptr, wkj_ptr, ss_ptr); // n*E[X^2]-n*E[X]^2
                // compute the likelihood of that position of the word
                probs_l[k] += t_likelihood (wkj_ptr, ss_ptr, (double) n,
                  l_ind_ptr[k]->vowel_ptr[j].phon_ptr->mu_ptr, l_ind_ptr[k]->vowel_ptr[j].phon_ptr->nu,
                  l_ind_ptr[k]->vowel_ptr[j].phon_ptr->varnu_ptr, matrix1_ptr, matrix2_ptr, vector1_ptr, p_ptr);
              }
            }
            probs_l[k] += log ((double) l_ind_ptr[k]->nl); // multiply by something proportional to the prior
            // keep a running count of the maximum, to subtract off before exponentiating
            if (probs_l[k] > *max_prob_ptr) {
              max_prob_ptr = probs_l+k; // pointer arithmetic for this log probability
            }
          }
        }

        // Make a table of probabilities
        double max = *max_prob_ptr; // to copy the value to a safe place
        max_prob_ptr = NULL; // for bookkeeping, and to avoid errors
        double totprob = 0.;
        for (k = 0; k < num_tables_l; k++) {
          if (cand[k] == 1) {
            probs_l[k] = exp (anneal * (probs_l[k] - max));
            totprob += probs_l[k];
          } else {
            probs_l[k] = 0.;
          }
        }
        for (k = 0; k < NUM_LEXICAL_SAMPLES; k++) {
          probs_l[num_tables_l+k] = exp (anneal * (probs_l[num_tables_l+k] - max));
          totprob += probs_l[num_tables_l+k];
        }
        
        // Sample a lexical item
        int assign = random_sample (probs_l, totprob, num_tables_l+NUM_LEXICAL_SAMPLES);
        if (assign >= num_tables_l) { // if it's a new lexical item
          l_ind_ptr[num_tables_l] = create_lexical_item (w+i, pseudo_word[assign-num_tables_l]);
          l_ind_ptr[num_tables_l]->index = num_tables_l+1; // set the index number
          // free all sampled vowel vectors except the one that was chosen
          for (j = 0; j < NUM_LEXICAL_SAMPLES; j++) {
            if (j != (assign - num_tables_l)) {
              free (pseudo_word[j]); // free the vowel vector
            }
          }
          assign = num_tables_l; // we've put the new table directly at the end of the restaurant
          num_tables_l++; // increment the number of tables in the lexicon
        } else {
        // free all sampled vowel vectors (none was chosen)
          for (j = 0; j < NUM_LEXICAL_SAMPLES; j++) {
            free (pseudo_word[j]); // free the vowel vector
          }
        }
        w[i].z_ptr = l_ind_ptr[assign];
        w[i].z_ptr->nl++; // increment the word count
        for (j = 0; j < w[i].vowel_length; j++) {
          // Update the parameters of the phoneme
          add_values (w[i].vowel_ptr[j].formant_ptr, zeros, 1., w[i].z_ptr->vowel_ptr[j].phon_ptr->mu_ptr, 
                           &w[i].z_ptr->vowel_ptr[j].phon_ptr->nu, w[i].z_ptr->vowel_ptr[j].phon_ptr->varnu_ptr, vector1_ptr); 
          // Update the parameters stored with the lexical item
          gsl_vector_add (w[i].z_ptr->vowel_ptr[j].wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // add the formant values for the phone
          gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, w[i].z_ptr->vowel_ptr[j].ss_ptr); // add sum of squares term
        }
      }
            
      /* END INITIALIZATION */
      
      
      
      
      int iteration;
      
      for (iteration = 0; iteration < MAX_ITER; iteration++) {
        
        printf ("Iteration %i: \n", iteration);
        
        //double anneal = 1;
        double anneal = (log ((double) iteration+2.)) / ANNEALING_CONSTANT;
        if (anneal > 1.) {
          anneal = 1.;
        }
        
        
        /* SWEEP TO RE-SAMPLE L */
        
        assert (l_ind_ptr[num_tables_l] == NULL);
        assert (c_ind_ptr[num_tables_c] == NULL);
        
        printf ("Sampling L...\n");
        // loop through all lexical items
        for (i = 0; i < num_tables_l; i++) {
          // loop through all vowels in the lexical item
          for (j = 0; j < l_ind_ptr[i]->vowel_length; j++) {
            // Correct the mean and sum-of-squares terms
            gsl_vector_memcpy (wkj_ptr, l_ind_ptr[i]->vowel_ptr[j].wkj_ptr);
            gsl_matrix_memcpy (ss_ptr, l_ind_ptr[i]->vowel_ptr[j].ss_ptr);
            gsl_vector_scale (wkj_ptr, 1. / (double) l_ind_ptr[i]->nl); // normalize the mean, E[X]
            gsl_blas_dger (-1.*(double) l_ind_ptr[i]->nl, wkj_ptr, wkj_ptr, ss_ptr); // n*E[X^2]-n*E[X]^2
            // Decrement the count of the current phoneme
            l_ind_ptr[i]->vowel_ptr[j].phon_ptr->nc--;
            // Delete the values from the phoneme
            subtract_values (wkj_ptr, ss_ptr, (double) l_ind_ptr[i]->nl, l_ind_ptr[i]->vowel_ptr[j].phon_ptr->mu_ptr, 
                             &l_ind_ptr[i]->vowel_ptr[j].phon_ptr->nu, l_ind_ptr[i]->vowel_ptr[j].phon_ptr->varnu_ptr, vector1_ptr);
            // If the phoneme is no longer used, delete the table
            if (l_ind_ptr[i]->vowel_ptr[j].phon_ptr->nc == 0) {
              //printf ("Destroying phoneme %i\n", l_ind_ptr[i]->vowel_ptr[j].phon_ptr->index);
              destroy_phoneme_index (l_ind_ptr[i]->vowel_ptr[j].phon_ptr); // delete it from the restaurant
              destroy_phoneme (l_ind_ptr[i]->vowel_ptr[j].phon_ptr); // and free the memory
              num_tables_c--;
            }
            l_ind_ptr[i]->vowel_ptr[j].phon_ptr = NULL;
            
            double probs[num_tables_c+1];
            double *max_prob_ptr = probs; // keep track of the highest probability, to subtract it off
            for (k = 0; k < num_tables_c; k++) {
              // compute log likelihood for all phonemes
              // and multiply by something proportional to the prior
              probs[k] = t_likelihood (wkj_ptr, ss_ptr, (double) l_ind_ptr[i]->nl, c_ind_ptr[k]->mu_ptr,
                                      c_ind_ptr[k]->nu, c_ind_ptr[k]->varnu_ptr, matrix1_ptr, matrix2_ptr,
                                      vector1_ptr, p_ptr);
              assert (probs[k] < 0);
              probs[k] += log ((double) c_ind_ptr[k]->nc);
              if (probs[k] > *max_prob_ptr) {
                max_prob_ptr = probs + k; // pointer arithmetic for this phonetic category probability
              }
            }
            // log probability of a new table
            probs[num_tables_c] = t_likelihood(wkj_ptr, ss_ptr, (double) l_ind_ptr[i]->nl, mu0_ptr, nu0,
                                               varnu0_ptr, matrix1_ptr, matrix2_ptr, vector1_ptr, p_ptr) 
                                               + log (beta);
            if (probs[num_tables_c] > *max_prob_ptr) {
              max_prob_ptr = probs + num_tables_c; // pointer arithmetic for this phonetic category probability
            }
            
            double max = *max_prob_ptr; // copy the value to a safe place
            max_prob_ptr = NULL; // for bookkeeping, and to avoid errors
            double totprob = 0.;
            for (k = 0; k <= num_tables_c; k++) {
              probs[k] = exp (anneal * (probs[k]-max));
              totprob += probs[k]; // normalization constant
            }
            
            // Sample a phonetic category
            int assign = random_sample (probs, totprob, num_tables_c+1);
            if (assign == num_tables_c) {
              //printf ("Creating phoneme %i\n", num_tables_c+1);
              c_ind_ptr[num_tables_c] = create_phoneme();
              c_ind_ptr[num_tables_c]->index = num_tables_c+1;
              num_tables_c++;
              c_ind_ptr[num_tables_c] = NULL;
            }
            l_ind_ptr[i]->vowel_ptr[j].phon_ptr = c_ind_ptr[assign];
            add_values (wkj_ptr, ss_ptr, (double) l_ind_ptr[i]->nl, l_ind_ptr[i]->vowel_ptr[j].phon_ptr->mu_ptr, 
                        &l_ind_ptr[i]->vowel_ptr[j].phon_ptr->nu, l_ind_ptr[i]->vowel_ptr[j].phon_ptr->varnu_ptr, 
                        vector1_ptr); // add values to new phoneme
            l_ind_ptr[i]->vowel_ptr[j].phon_ptr->nc++; // increment the phoneme count
          }
        }
        
        /* END SWEEP TO RE-SAMPLE L */
        
        /* SWEEP TO RE-SAMPLE Z */
        
        printf ("Sampling Z...\n");
        // loop through all words in the corpus
        for (i = 0; i < NUM_WORDS; i++) {
          w[i].z_ptr->nl--;
          for (j = 0; j < w[i].vowel_length; j++) {
            // Remove the contribution of this value from its phoneme parameters
            subtract_values (w[i].vowel_ptr[j].formant_ptr, zeros, 1., w[i].z_ptr->vowel_ptr[j].phon_ptr->mu_ptr, 
                             &w[i].z_ptr->vowel_ptr[j].phon_ptr->nu, w[i].z_ptr->vowel_ptr[j].phon_ptr->varnu_ptr,
                             vector1_ptr);
            // Remove the contribution of this value from its lexical item parameters
            gsl_vector_sub (w[i].z_ptr->vowel_ptr[j].wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // subtract the formant values for the phone
            gsl_blas_dger (-1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, w[i].z_ptr->vowel_ptr[j].ss_ptr); // subtract ss term        
          }
          // if lexical item is no longer used, delete the table but use as part of sample from the prior
          if (w[i].z_ptr->nl == 0) {
            pseudo_word[0] = w[i].z_ptr->vowel_ptr;
            destroy_lexical_item (w[i].z_ptr);
            num_tables_l--;
            double probs_c[num_tables_c+1];
            for (j = 0; j < num_tables_c; j++) {
              probs_c[j] = (double) c_ind_ptr[j]->nc;
            }
            probs_c[num_tables_c] = beta;
            // sample the rest of the new lexical items
            for (j = 1; j < NUM_LEXICAL_SAMPLES; j++) {
              pseudo_word[j] = sample_lexical_item (probs_c, w+i, num_tables_c+1, anneal);
            }
          } else {
            // Sample the lexical items
            double probs_c[num_tables_c+1];
            for (j = 0; j < num_tables_c; j++) {
              probs_c[j] = (double) c_ind_ptr[j]->nc;
            }
            probs_c[num_tables_c] = beta;
            for (j = 0; j < NUM_LEXICAL_SAMPLES; j++) {
              pseudo_word[j] = sample_lexical_item(probs_c, w+i, num_tables_c+1, anneal);
            }
          }
          
          w[i].z_ptr = NULL; // avoid bookkeeping errors
          
          //printf ("Calculating alpha...\n");
          // Figure out the effective value of alpha for this lexical frame
          // Divide by the number of samples from the prior
          double log_eff_alpha = log (alpha) - log ((double) NUM_LEXICAL_SAMPLES);
          // Term for length: g^(length-1) * (1-g)
          log_eff_alpha += ((double) w[i].word_length-1.) * log (GEOMETRIC);
          log_eff_alpha += log (1.-GEOMETRIC);
          // Term for syllable template: p_cons^num_cons * (1-p_cons)^num_vowels
          log_eff_alpha += (double) w[i].cons_length * log (CONS);
          log_eff_alpha += (double) w[i].vowel_length * log (1-CONS);
          // Term for particular consonants
          int my_ncons[NUM_CONSONANTS], total_ncons = 0;
          for (j = 0; j < NUM_CONSONANTS; j++) {
            my_ncons[j] = 0; // initialize the array
            total_ncons += ncons[j]; // meanwhile, compute the total number of consonants in the lexicon
          }
          for (j = 0; j < w[i].cons_length; j++) {
            my_ncons[w[i].cons_ptr[j]-1]++; // consonants numbered from 1, but array indexed from 0
            // divide by the denominator total_ncons * total_ncons+1 * ... * total_ncons+cons_length-1
            log_eff_alpha -= log ((double) total_ncons + (double) j);
          }
          for (j = 0; j < NUM_CONSONANTS; j++) {
            // numerator is ncons[j] * ncons[j]+1 * ... * ncons[j]+my_ncons[j]-1
            if (my_ncons[j] > 0) {
              if (ncons[j] > 0) {
                log_eff_alpha += log ((double) ncons[j]);
              } else {
                log_eff_alpha += log (beta);
              }
              // will only go through this loop if my_ncons[j]>1
              for (k = 1; k < my_ncons[j]; k++) {
                log_eff_alpha += log ((double) ncons[j]+(double) k);
              }
            }
          }
          
          // Figure out which lexical items are candidates for this word
          int cand[num_tables_l];
          for (k = 0; k < num_tables_l; k++) {
            // Initially assume it is a candidate
            cand[k] = 1;
            // Check whether the word is the right length
            if (w[i].word_length != l_ind_ptr[k]->lex_length) { 
              cand[k] = 0;
            } else {
              // If the right length, check the syllable template
              for (j = 0; j < l_ind_ptr[k]->lex_length; j++) {
                if (w[i].CV_ptr[j] != l_ind_ptr[k]->CV_ptr[j]) { 
                  cand[k] = 0;
                  break; // break out of the for loop, don't check any more CV slots
                }
              }
            }
            // If it's still a candidate, check the consonant identities
            if (cand[k] == 1) {
              for (j = 0; j < l_ind_ptr[k]->cons_length; j++) {
                if (w[i].cons_ptr[j] != l_ind_ptr[k]->cons_ptr[j]) {
                  cand[k] = 0;
                  break; // break out of the for loop, don't check any more consonant slots
                }
              }
            }
          }
          
          // Construct the table of probabilities
          double probs_l[num_tables_l+NUM_LEXICAL_SAMPLES];
          double *max_prob_ptr = probs_l+num_tables_l; // keep track of the highest probability, to subtract it off
          
          // Indicator vector to say whether phonemes are duplicated within a lexical item
          int duplicate[w[i].vowel_length]; 
          
          //printf ("Likelihood of newly sampled tables...\n");
          // Compute the log likelihood of newly sampled tables
          for (k = 0; k < NUM_LEXICAL_SAMPLES; k++) {
            probs_l[num_tables_l+k] = 0.;
            // Initialize the duplicate-indicator vector
            for (j = 0; j < w[i].vowel_length; j++) {
              duplicate[j] = 0;
            }
            for (j = 0; j < w[i].vowel_length; j++) {
              if (duplicate[j] == 0) { // if it's not a duplicate from before
                // Search through to see whether there are duplicate phonemes in the word
                gsl_vector_memcpy (wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // initialize wkj to formant value
                gsl_matrix_memcpy (ss_ptr, zeros);
                gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, ss_ptr); // initialize ss term
                int n = 1; // number of times phoneme occurs in the word
                int l;
                for (l = j+1; l < w[i].vowel_length; l++) {
                  if (pseudo_word[k][j].phon_ptr == pseudo_word[k][l].phon_ptr) {
                    duplicate[l] = 1;
                    // Combine the parameters of the duplicates
                    gsl_vector_add (wkj_ptr, w[i].vowel_ptr[l].formant_ptr);
                    gsl_blas_dger (1., w[i].vowel_ptr[l].formant_ptr, w[i].vowel_ptr[l].formant_ptr, ss_ptr);
                    n++;
                  }
                }
                gsl_vector_scale (wkj_ptr, 1./ (double) n); // normalize the mean, E[X]
                gsl_blas_dger (-1.*(double) n, wkj_ptr, wkj_ptr, ss_ptr); // n*E[X^2]-n*E[X]^2
                // compute the likelihood of the acoustic values for that phoneme
                probs_l[num_tables_l+k] += t_likelihood (wkj_ptr, ss_ptr, (double) n,
                  pseudo_word[k][j].phon_ptr->mu_ptr, pseudo_word[k][j].phon_ptr->nu,
                  pseudo_word[k][j].phon_ptr->varnu_ptr, matrix1_ptr, matrix2_ptr, vector1_ptr, p_ptr);
              }
            }
            probs_l[num_tables_l+k] += log_eff_alpha; // multiply by something proportional to the prior
            // keep a running count of the maximum, to subtract off before exponentiating
            if (probs_l[num_tables_l+k] > *max_prob_ptr) {
              max_prob_ptr = probs_l+num_tables_l+k; // pointer arithmetic for this log probability
            }
          }
          
          // Compute the log likelihood of existing tables
          for (k = 0; k < num_tables_l; k++) {
            if (cand[k] == 1) { // cand[k] will be 1 for candidates, 0 otherwise
              probs_l[k] = 0.;
              // Initialize the duplicate-indicator vector
              for (j = 0; j < w[i].vowel_length; j++) {
                duplicate[j] = 0;
              }
              for (j = 0; j < w[i].vowel_length; j++) {
                if (duplicate[j] == 0) { // if it's not a duplicate from before
                  // Search through to see whether there are duplicate phonemes in the word
                  gsl_vector_memcpy (wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // initialize wkj to formant value
                  gsl_matrix_memcpy (ss_ptr, zeros);
                  gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, ss_ptr); // initialize ss term
                  int n = 1; // number of times phoneme occurs in the word
                  int l;
                  for (l = j+1; l < w[i].vowel_length; l++) {
                    if (l_ind_ptr[k]->vowel_ptr[j].phon_ptr == l_ind_ptr[k]->vowel_ptr[l].phon_ptr) {
                      duplicate[l] = 1;
                      // Combine the parameters of the duplicates
                      gsl_vector_add (wkj_ptr, w[i].vowel_ptr[l].formant_ptr);
                      gsl_blas_dger (1., w[i].vowel_ptr[l].formant_ptr, w[i].vowel_ptr[l].formant_ptr, ss_ptr);
                      n++;
                    }
                  }
                  gsl_vector_scale (wkj_ptr, 1./ (double) n); // normalize the mean, E[X]
                  gsl_blas_dger (-1.*(double) n, wkj_ptr, wkj_ptr, ss_ptr); // n*E[X^2]-n*E[X]^2
                  // compute the likelihood of that position of the word
                  probs_l[k] += t_likelihood (wkj_ptr, ss_ptr, (double) n,
                    l_ind_ptr[k]->vowel_ptr[j].phon_ptr->mu_ptr, l_ind_ptr[k]->vowel_ptr[j].phon_ptr->nu,
                    l_ind_ptr[k]->vowel_ptr[j].phon_ptr->varnu_ptr, matrix1_ptr, matrix2_ptr, vector1_ptr, p_ptr);
                }
              }
              probs_l[k] += log ((double) l_ind_ptr[k]->nl); // multiply by something proportional to the prior
              // keep a running count of the maximum, to subtract off before exponentiating
              if (probs_l[k] > *max_prob_ptr) {
                max_prob_ptr = probs_l+k; // pointer arithmetic for this log probability
              }
            }
          }

          // Make a table of probabilities
          double max = *max_prob_ptr; // to copy the value to a safe place
          max_prob_ptr = NULL; // for bookkeeping, and to avoid errors
          double totprob = 0.;
          for (k = 0; k < num_tables_l; k++) {
            if (cand[k] == 1) {
              probs_l[k] = exp (anneal * (probs_l[k] - max));
              totprob += probs_l[k];
            } else {
              probs_l[k] = 0.;
            }
          }
          for (k = 0; k < NUM_LEXICAL_SAMPLES; k++) {
            probs_l[num_tables_l+k] = exp (anneal * (probs_l[num_tables_l+k] - max));
            totprob += probs_l[num_tables_l+k];
          }
          
          // Sample a lexical item
          int assign = random_sample (probs_l, totprob, num_tables_l+NUM_LEXICAL_SAMPLES);
          if (assign >= num_tables_l) { // if it's a new lexical item
            l_ind_ptr[num_tables_l] = create_lexical_item (w+i, pseudo_word[assign-num_tables_l]);
            l_ind_ptr[num_tables_l]->index = num_tables_l+1; // set the index number
            // free all sampled vowel vectors except the one that was chosen
            for (j = 0; j < NUM_LEXICAL_SAMPLES; j++) {
              if (j != (assign - num_tables_l)) {
                free (pseudo_word[j]); // free the vowel vector
              }
            }
            assign = num_tables_l; // we've put the new table directly at the end of the restaurant
            num_tables_l++; // increment the number of tables in the lexicon
          } else {
          // free all sampled vowel vectors (none was chosen)
            for (j = 0; j < NUM_LEXICAL_SAMPLES; j++) {
              free (pseudo_word[j]); // free the vowel vector
            }
          }
          w[i].z_ptr = l_ind_ptr[assign];
          w[i].z_ptr->nl++; // increment the word count
          for (j = 0; j < w[i].vowel_length; j++) {
            // Update the parameters of the phoneme
            add_values (w[i].vowel_ptr[j].formant_ptr, zeros, 1., w[i].z_ptr->vowel_ptr[j].phon_ptr->mu_ptr, 
                             &w[i].z_ptr->vowel_ptr[j].phon_ptr->nu, w[i].z_ptr->vowel_ptr[j].phon_ptr->varnu_ptr, vector1_ptr); 
            // Update the parameters stored with the lexical item
            gsl_vector_add (w[i].z_ptr->vowel_ptr[j].wkj_ptr, w[i].vowel_ptr[j].formant_ptr); // add the formant values for the phone
            gsl_blas_dger (1., w[i].vowel_ptr[j].formant_ptr, w[i].vowel_ptr[j].formant_ptr, w[i].z_ptr->vowel_ptr[j].ss_ptr); // add sum of squares term
          }
        }
        
        /* END SWEEP TO RE-SAMPLE Z */
        
        
        if ((iteration+1) % LIKELIHOOD_FREQ == 0) {
          // Refresh parameters, and compute joint probability
          double ll = joint_probability(matrix1_ptr, matrix2_ptr, vector1_ptr, p_ptr);
          printf ("Iteration %i: %g\n", iteration, ll);
          filewrite(iteration, ll);
        }
        printf ("%i lexical items, %i phonemes\n", num_tables_l, num_tables_c);
      }
      
      
      /* CLEAN-UP */
      
      while (l_ind_ptr[0] != NULL) {
        phoneme_slot *temp_ptr = l_ind_ptr[0]->vowel_ptr;
        destroy_lexical_item (l_ind_ptr[0]);
        free (temp_ptr); // free the vowel vector
      }
      assert (num_tables_c == 0);
      
      gsl_vector_free (wkj_ptr);
      gsl_matrix_free (ss_ptr);
      
  
    }
  
  }
  
  
  
  for (i = 0; i < LONGEST; i++) {
    assert (extra_phonemes_ptr[i]->nc == 0);
    destroy_phoneme (extra_phonemes_ptr[i]);
  }
  
  free_w();

  gsl_matrix_free (matrix1_ptr);
  gsl_matrix_free (matrix2_ptr);
  gsl_vector_free (vector1_ptr);
  gsl_permutation_free (p_ptr);

  gsl_vector_free (mu0_ptr);
  gsl_matrix_free (varnu0_ptr);
  gsl_matrix_free (zeros);
  
  /* END CLEAN-UP */

  return 0; // tell the user the program was successful

}